#include <stdio.h>
#include <utmpx.h>
#include <pwd.h>
#include <sys/stat.h>

#define  USER   	1
#define  HOST   	2
#define  HOST_USER      3

#define  FAIL		0
#define  SUCCESS	1

struct	idle_time_t	{
	int	days;
	int	hour;
	int	minute;
};

struct  user_t  {
	char	*login_name;
	char	*full_name;
	char	*home_directory;
	char	*shell;
	char	*login_time;
	char	*tty;
	char	*host;
	struct	idle_time_t	*idle;
	char	*mail;
};

main(int argc, char *argv[])
{
        int     check_argument(char *);
        int	get_user_information(struct utmpx *, char *, struct user_t *);
        void    print_local_information();
        void    print_user(struct user_t *);

        struct  user_t  *user;
        struct  host_t  *host;
	struct	utmpx	*utmpx;
        char    argument[64];
        int     result = 0;
	int	information = FAIL;


        strcpy(argument, "");
        if(*++argv)
                strcpy(argument, *argv);
        if(strcmp(argument , ""))
                result = check_argument(argument);
	user = (struct user_t *)malloc(sizeof(struct user_t));
	while((utmpx = getutxent()) != NULL)
	if(result == USER)
		{
		information = get_user_information(utmpx, argument, user);
		if(information == FAIL) continue;
		print_user(user);
		information = SUCCESS;
		}
	else    if(result == HOST)
		{
		}
	else    if(result == HOST_USER)
		{
		}
	else	print_local_information(utmpx);

}

int
check_argument(char *argument)
{
        if(!strncmp(argument, "@", 1))
		return HOST;
        else    if(strchr(argument, '@'))
             	return HOST_USER;
        else    return USER;
}

int
get_user_information(struct utmpx *utmpx, char *user_name, struct user_t *user)
{
	void	get_idle_time(struct utmpx *, struct idle_time_t *);
	void	get_mailbox(struct user_t *, char *);
	struct	passwd	*pass;
	int	len;


	user->idle = (struct idle_time_t *)malloc(sizeof(struct idle_time_t));
	user->idle->days = user->idle->hour = user->idle->minute = 0;

	if(utmpx->ut_type != USER_PROCESS)
		return FAIL;
	if(strcmp(utmpx->ut_user, user_name))
		return FAIL;
	if((pass = getpwnam(utmpx->ut_user)) != NULL)
		user->full_name =  pass->pw_gecos;
	else	sprintf(user->full_name, "%d", pass->pw_uid);
	user->login_name = utmpx->ut_user;
	user->home_directory = pass->pw_dir;
	user->shell = pass->pw_shell;
	user->login_time = ctime(&utmpx->ut_tv.tv_sec);
	len = strlen(user->login_time);
	user->login_time[len - 1] = '\0';
	user->tty = utmpx->ut_line;
	if(utmpx->ut_syslen > 0)
		user->host = utmpx->ut_host;
	else	user->host = "unknown";
	get_idle_time(utmpx, user->idle);
	get_mailbox(user, user->mail);
	
	return SUCCESS;

}

void
get_idle_time(struct utmpx *utmpx, struct idle_time_t *idle_time)
{
	void	fatal(char *);
	int	minute, hour, days;
	int	idle_minute, idle_hour;
	time_t	idle_second;
	time_t	current;
	char	device_name[20];
	struct	stat device_stat;


	minute = hour = days = 0;
	idle_minute = idle_hour = 0;

	strcpy(device_name, "/dev/");
	strcat(device_name, utmpx->ut_line);
	if(stat(device_name, &device_stat) < 0)
		fatal("stat");

	time(&current);
	if(current < device_stat.st_atime)
		idle_second = 0L;
	else	idle_second = current - device_stat.st_atime;

	minute = idle_second / 60;

	if(minute)
		hour = minute / 60;
	else	minute = 0;

	if(hour)
		{
		idle_minute = minute % 60;
		days = hour / 24;
		}
	else	idle_minute = minute;

	if(days)
		idle_hour = hour % 24;
	else	idle_hour = hour;

	if(days)
		idle_time->days =  days;
	if(hour)
		idle_time->hour = idle_hour;
	if(minute)
		idle_time->minute =  idle_minute;
}

void
get_mailbox(struct user_t *user, char *mail)
{
	struct	stat mail_stat;
	char	mail_path[30];


	strcat(mail_path, user->login_name);
	if(stat(mail_path, &mail_stat) < 0 || mail_stat.st_size == 0)
		user->mail = "No unread mail";
	else	user->mail = "New mail received";
}
	

void
print_user(struct user_t *user)
{
	static	int first_check;


	printf("Login name: %s\t\tIn real Life: %s\n", user->login_name, user->full_name);
	printf("Directory: %s\t\tShell: %s\n", user->home_directory, user->shell);
	printf("On since %s on %s from %s\n", user->login_time, user->tty, user->host);
	if(user->idle->days)
		printf("%d days ", user->idle->days);
	if(user->idle->hour)
		printf("%d hours ", user->idle->hour);
	if(user->idle->minute)
		printf("%d minute Idle Time\n", user->idle->minute);
	if(first_check == 0)
		puts(user->mail);
	first_check = 1;

	printf("\n\n");
}

void
print_local_information(struct utmpx *utmpx)
{
	int	get_user_information(struct utmpx *, char *, struct user_t *);
	struct	user_t *user;
	struct	tm *old;
	char	*week[7] =
		{"Sun", "Mon", "Tue", "Wed", "Thur", "Fri", "Sat"};
	static	int first_check = 0;

	if(utmpx->ut_type != USER_PROCESS)
		return;

	user = (struct user_t *)malloc(sizeof(struct user_t));
	if(get_user_information(utmpx, utmpx->ut_name, user) == FAIL)
		return;
	if(user->idle->hour == 0 && user->idle->minute == 0)
		return;
	if(!strcmp(user->host, "unknown"))
		return;
	if(first_check == 0)
		printf("%-12s %-10s %-9s %-6s %-7s %s\n",
		"Login", "Name", "TTY", "Idle", "When", "Where");
	printf("%-12s %-10s %-10s", 
		user->login_name, user->full_name, user->tty);
	if(user->idle->hour)
		printf("%d:", user->idle->hour);
	else	printf("   ");
	if(user->idle->minute)
		printf("%02d", user->idle->minute);
	old = localtime(&utmpx->ut_tv.tv_sec);
	printf(" %s %d:%02d %s\n", week[old->tm_wday], old->tm_hour, old->tm_min, user->host);
	first_check = 1;
	
}

void
fatal(char *error_name)
{
	perror(error_name);
	exit(1);
}
